<?php
namespace Supportingcast;

use Requests;

ApiTokenOauth::boot();

class ApiTokenOauth
{

	const CLIENT_ID = 200;

	const STATUS_NO_OP = 0;
	const STATUS_CANCELLED = 1;
	const STATUS_COMPLETE_OK = 2;
	const STATUS_COMPLETE_FAILED = 3;

	static $status = self::STATUS_NO_OP;

    public static function boot()
    {
        $apiTokenOauth = new ApiTokenOauth();

	    add_action('init', [$apiTokenOauth, 'oauth_redirect']);
	    add_action('init', [$apiTokenOauth, 'oauth_complete']);

    }

    public function oauth_redirect() {

    	if (!isset($_POST['SC_OAUTH_REDIRECT']) || empty($_POST['SC_OAUTH_REDIRECT'])) {
    		return;
	    }

	    $domain = $_POST['SC_OAUTH_REDIRECT'];
    	if (strpos($domain, '.') === false) {
    		$domain .= '.supportingcast.fm';
	    }

	    $state = wp_generate_password(40, false);
	    $codeVerifier = wp_generate_password(128, false);
	    $redirectUri = $this->getRedirectUri();



	    // save to authenticate when they return
	    update_user_meta(get_current_user_id(),
		    'sc_oauth_request',
		    [
			    'state' => $state,
			    'code_verifier' => $codeVerifier,
			    'redirect_uri' => $redirectUri,
			    'domain' => $domain,
		    ]
	    );

	    $codeChallenge = strtr(rtrim(
		    base64_encode(hash('sha256', $codeVerifier, true))
		    , '='), '+/', '-_');

	    $query = http_build_query([
		    'client_id' => self::CLIENT_ID,
		    'redirect_uri' => $redirectUri,
		    'response_type' => 'code',
		    'scope' => '',
		    'state' => $state,
		    'code_challenge' => $codeChallenge,
		    'code_challenge_method' => 'S256',
	    ]);

	    $this->redirect('https://'.$domain.'/admin/oauth/authorize?'.$query);
    }

    public function oauth_complete() {

	    if (!isset($_GET['SC_OAUTH_COMPLETE']) || empty($_GET['SC_OAUTH_COMPLETE'])) {
		    return;
	    }

	    if (isset($_GET['error'])) {
		    $this->redirect("/wp-admin/options-general.php?" . http_build_query(['page' => 'supporting-cast', 'oauth_status' => $_GET['error']]) );
	    }

	    $failHelp = '<br>Please make sure no one else is attempting to get an API token using your WP user and '
	               . '<a href="/wp-admin/options-general.php?page=supporting-cast">try again.</a>';

	    self::$status = self::STATUS_COMPLETE_FAILED;

	    $oauthRequest = get_user_meta(get_current_user_id(), 'sc_oauth_request', true);

	    if ($oauthRequest['state'] != $_GET['state']) {
	    	// This happens if another request is started and the 'state' does not match.
	    	wp_die('Oauth State mismatch, this token is from another request. ' . $failHelp );
	    }

	    $requestData = [
		    'grant_type' => 'authorization_code',
		    'client_id' => self::CLIENT_ID,
		    'redirect_uri' => $oauthRequest['redirect_uri'],
		    'code_verifier' => $oauthRequest['code_verifier'],
		    'code' => $_GET['code'],
	    ];


	    $response = Requests::post(
		    "https://{$oauthRequest['domain']}/admin/oauth/token",
		    $this->getHeaders(),
		    http_build_query($requestData),
		    $this->getOptions()
	    );

	    $json = json_decode((string) $response->body, true);

		if (!$response->success) {
			wp_die("Failed to convert oauth access code to API token (Request failed - {$response->status_code} {$json['message']})" . $failHelp);
		}

		if (!isset($json['access_token']) || empty($json['access_token']) ) {
			wp_die("Failed to convert oauth access code to API token (No token in response)" . $failHelp);
		}

	    update_option('sc-token', $json['access_token'], true);
	    update_option('sc-token-refresh', $json['refresh_token'], false);
	    update_option('sc-token-expire', $json['expires_in'], false);

	    self::$status = self::STATUS_COMPLETE_OK;

	    $this->redirect("/wp-admin/options-general.php?page=supporting-cast&oauth_status=success");

    }

    public static function getStatusMessage($code) {
    	$msgs = [
    		'bad_domain' => _("The Supporting Cast domain you entered is not valid, please try again or contact your account representative for more help."),
    		'success' => _('A new API token has been successfully acquired.'),
    		'access_denied' => _("It looks like you pressed the <b>Cancel</b> button. You must press the authorize button to get an API token."),
	    ];

    	return isset($msgs[$code]) ? $msgs[$code] : "Error getting oauth token: " . $code;
    }

    private function redirect($url) {
	    if (wp_redirect($url)) {
		    exit;
	    }

	    wp_die("Redirect cancelled by WP");
    }

	private function getRedirectUri() {

		$port = 80;
		$url = "http://" . $_SERVER['SERVER_NAME'];

		if (isset($_SERVER['HTTPS']) && !empty($_SERVER['HTTPS'])) {
			$port = 443;
			$url = "https://" . $_SERVER['SERVER_NAME'];
		}

		if ($_SERVER['SERVER_PORT'] != $port) {
			$url .= ':' . $_SERVER['SERVER_PORT'];
		}

		$url .= '/wp-admin/options-general.php?page=supporting-cast&SC_OAUTH_COMPLETE=true';

		return $url;
	}

	// for new Request:: calls
	private function getOptions() {
		return [
			'verify' => false,
			'timeout' => 30,
		];
	}

	private function getHeaders() {
		return [
			//'Content-type' => 'application/json',
			'Accept' => 'application/json',
		];
	}
}